// -*- C++ -*-
//
// This file is part of LHAPDF
// Copyright (C) 2012-2024 The LHAPDF collaboration (see AUTHORS for details)
//
#pragma once
#ifndef LHAPDF_LHAPDF_H
#define LHAPDF_LHAPDF_H

/// @file LHAPDF.h
/// Just a convenience header that pulls in the main functionality with a single include

#include "LHAPDF/Version.h"
#include "LHAPDF/PDF.h"
#include "LHAPDF/PDFSet.h"
#include "LHAPDF/PDFInfo.h"
#include "LHAPDF/Factories.h"
#include "LHAPDF/PDFIndex.h"
#include "LHAPDF/Paths.h"
#include "LHAPDF/LHAGlue.h"

#endif
